@@ -1,7 +1,7 @@
 /*
  * @name modcall.c
  *
- * Version:	$Id: 4723971d20f7fbb7a369c8c28c073d88df6a5983 $
+ * Version:	$Id: 5a3116cc58cd6ed3dd25c23f418a1e0357da121a $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000,2006  The FreeRADIUS server project
  */
 
-RCSID("$Id: 4723971d20f7fbb7a369c8c28c073d88df6a5983 $")
+RCSID("$Id: 5a3116cc58cd6ed3dd25c23f418a1e0357da121a $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modpriv.h>
@@ -276,6 +276,7 @@ static rlm_rcode_t CC_HINT(nonnull) call_modsingle(rlm_components_t component, m
 {
 	int blocked;
 	int indent = request->log.indent;
+	char const *old;
 
 	/*
 	 *	If the request should stop, refuse to do anything.
@@ -296,20 +297,21 @@ static rlm_rcode_t CC_HINT(nonnull) call_modsingle(rlm_components_t component, m
 	/*
 	 *	For logging unresponsive children.
 	 */
+	old = request->module;
 	request->module = sp->modinst->name;
 
 	safe_lock(sp->modinst);
 	request->rcode = sp->modinst->entry->module->methods[component](sp->modinst->insthandle, request);
 	safe_unlock(sp->modinst);
 
-	request->module = "";
+	request->module = old;
 
 	/*
 	 *	Wasn't blocked, and now is.  Complain!
 	 */
 	blocked = (request->master_state == REQUEST_STOP_PROCESSING);
 	if (blocked) {
-		RWARN("Module %s became unblocked", sp->modinst->entry->name);
+		RWARN("Module %s(%s) became unblocked", sp->modinst->name, sp->modinst->entry->name);
 	}
 
  fail:
@@ -480,7 +482,11 @@ redo:
 
 		RDEBUG2("%s %s{", unlang_keyword[c->type], c->name);
 
-		condition = radius_evaluate_cond(request, result, 0, g->cond);
+		/*
+		 *	Use "result" UNLESS it wasn't set, in which
+		 *	case we use the previous result on the stack.
+		 */
+		condition = radius_evaluate_cond(request, result != RLM_MODULE_UNKNOWN ? result : entry->result, 0, g->cond);
 		if (condition < 0) {
 			condition = false;
 			REDEBUG("Failed retrieving values required to evaluate condition");
@@ -1510,7 +1516,6 @@ static const int authtype_actions[GROUPTYPE_COUNT][RLM_MODULE_NUMCODES] =
 int modcall_fixup_update(vp_map_t *map, UNUSED void *ctx)
 {
 	CONF_PAIR *cp = cf_item_to_pair(map->ci);
-
 	/*
 	 *	Anal-retentive checks.
 	 */
@@ -1587,7 +1592,7 @@ int modcall_fixup_update(vp_map_t *map, UNUSED void *ctx)
 		}
 
 		/*
-		 *	Only += and :=, and !* operators are supported
+		 *	Only += and :=, and !*, and ^= operators are supported
 		 *	for lists.
 		 */
 		switch (map->op) {
@@ -1621,6 +1626,14 @@ int modcall_fixup_update(vp_map_t *map, UNUSED void *ctx)
 			}
 			break;
 
+		case T_OP_PREPEND:
+			if ((map->rhs->type != TMPL_TYPE_LIST) &&
+			    (map->rhs->type != TMPL_TYPE_EXEC)) {
+				cf_log_err(map->ci, "Invalid source for list assignment '%s ^= ...'", map->lhs->name);
+				return -1;
+			}
+			break;
+
 		default:
 			cf_log_err(map->ci, "Operator \"%s\" not allowed for list assignment",
 				   fr_int2str(fr_tokens, map->op, "<INVALID>"));
@@ -1799,6 +1812,14 @@ static modcallable *do_compile_modswitch (modcallable *parent, rlm_components_t
 		return NULL;
 	}
 
+	/*
+	 *	Warn about confusing things.
+	 */
+	if ((vpt->type == TMPL_TYPE_ATTR) && (*name2 != '&')) {
+		WARN("%s[%d]: Please change \"switch %s\" to \"switch &%s\"",
+		     cf_section_filename(cs), cf_section_lineno(cs),
+		     name2, name2);
+	}
 
 	/*
 	 *	Walk through the children of the switch section,
